/*
 * Copyright (c) 2017 Intel Corporation.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

//#define LOG_LEVEL CONFIG_LOG_DEFAULT_LEVEL
//#include <logging/log.h>
//LOG_MODULE_REGISTER(main);

#include <zephyr.h>
#include <sys/printk.h>
#include <string.h>
#include <stdio.h>
//#include <ztest.h>

#include <drivers/spi.h>

#define SPI_DRV_NAME	"SPI_3"
#define SPI_SLAVE	0
#define FAST_FREQ	16000000

#define CS_CTRL_GPIO_DRV_NAME "GPIO_0"
struct spi_cs_control spi_cs = {
	.gpio_pin = 28,
	.gpio_dt_flags = GPIO_ACTIVE_LOW,
	.delay = 0,
};
#define SPI_CS (&spi_cs)


/* to run this test, connect MOSI pin to the MISO of the SPI */

#define STACK_SIZE 512
#define BUF_SIZE 17
#define BUF2_SIZE 36


/* this src memory shall be in RAM to support using as a DMA source pointer.*/
uint8_t buffer_tx[] = "0123456789abcdef\0";
uint8_t buffer_rx[BUF_SIZE] = {};


struct spi_config spi_cfg_fast = {
	.frequency = FAST_FREQ,
#if CONFIG_SPI_LOOPBACK_MODE_LOOP
	.operation = SPI_OP_MODE_MASTER | SPI_MODE_CPOL | SPI_MODE_LOOP |
#else
	.operation = SPI_OP_MODE_MASTER | SPI_MODE_CPOL |
#endif
	SPI_MODE_CPHA | SPI_WORD_SET(8) | SPI_LINES_SINGLE,
	.slave = SPI_SLAVE,
	.cs = SPI_CS,
};

static int cs_ctrl_gpio_config(void)
{
	spi_cs.gpio_dev = device_get_binding(CS_CTRL_GPIO_DRV_NAME);
	if (!spi_cs.gpio_dev) {
		printk("Cannot find %s!\n", CS_CTRL_GPIO_DRV_NAME);
		//zassert_not_null(spi_cs.gpio_dev, "Invalid gpio device");
	}

	return 0;
}

static struct k_poll_signal async_sig = K_POLL_SIGNAL_INITIALIZER(async_sig);
static struct k_poll_event async_evt =
	K_POLL_EVENT_INITIALIZER(K_POLL_TYPE_SIGNAL,
				 K_POLL_MODE_NOTIFY_ONLY,
				 &async_sig);
static K_SEM_DEFINE(caller, 0, 1);
K_THREAD_STACK_DEFINE(spi_async_stack, STACK_SIZE);
static int result = 1;

static void spi_async_call_cb(struct k_poll_event *async_evt,
			      struct k_sem *caller_sem,
			      void *unused)
{
	int ret;

	

	while (1) {
		printk("Polling...\n");
		ret = k_poll(async_evt, 1, K_FOREVER);
		result = async_evt->signal->result;
		k_sem_give(caller_sem);

		/* Reinitializing for next call */
		async_evt->signal->signaled = 0U;
		async_evt->state = K_POLL_STATE_NOT_READY;
	}
}

static int spi_async_call(const struct device *dev,
			  struct spi_config *spi_conf)
{
	const struct spi_buf tx_bufs[] = {
		{
			.buf = buffer_tx,
			.len = BUF_SIZE,
		},
	};
	const struct spi_buf rx_bufs[] = {
		{
			.buf = buffer_rx,
			.len = BUF_SIZE,
		},
	};
	const struct spi_buf_set tx = {
		.buffers = tx_bufs,
		.count = ARRAY_SIZE(tx_bufs)
	};
	const struct spi_buf_set rx = {
		.buffers = rx_bufs,
		.count = ARRAY_SIZE(rx_bufs)
	};
	int ret;

	printk("Start async call\n");

	ret = spi_transceive_async(dev, spi_conf, &tx, &rx, &async_sig);
	if (ret == -ENOTSUP) {
		printk("Not supported\n");
	}

	if (ret) {
		printk("spi_transceive_async failed %d\n", ret);
	}

	k_sem_take(&caller, K_FOREVER);

	if (result)  {
		printk("async_evt->signal returned %d\n", ret);
	}
	printk("Sent: '%s' |||  Received: '%s'\n", buffer_tx, buffer_rx);

	return 0;
}


void spi_loopback(void)
{
	
	const struct device *spi_fast;

	printk("SPI test on buffers TX/RX %p/%p\n", buffer_tx, buffer_rx);

	if (cs_ctrl_gpio_config()) {
		return;
	}

	spi_fast = device_get_binding(SPI_DRV_NAME);
	if (!spi_fast) {
		printk("Cannot find %s!\n", SPI_DRV_NAME);
	}

	spi_async_call(spi_fast, &spi_cfg_fast);

}

void main(void)
{
	struct k_thread async_thread;
	k_tid_t async_thread_id;
	async_thread_id = k_thread_create(&async_thread,
					spi_async_stack, STACK_SIZE,
					(k_thread_entry_t)spi_async_call_cb,
					&async_evt, &caller, NULL,
					K_PRIO_COOP(7), 0, K_NO_WAIT);
	while(true){
		spi_loopback();
		k_sleep(K_MSEC(2000));
	}
}
